"""
StarGAN v2
Copyright (c) 2020-present NAVER Corp.

This work is licensed under the Creative Commons Attribution-NonCommercial
4.0 International License. To view a copy of this license, visit
http://creativecommons.org/licenses/by-nc/4.0/ or send a letter to
Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.
"""

import os
import torch


class CheckpointIO(object):
    def __init__(self, fname_template, **kwargs):
        os.makedirs(os.path.dirname(fname_template), exist_ok=True)
        self.fname_template = fname_template
        self.module_dict = kwargs

    def register(self, **kwargs):
        self.module_dict.update(kwargs)

    def save_label(self, label_dict):
        print('Saving domain label configuration...')
        torch.save(label_dict, self.fname_template)

    def load_label(self):
        return torch.load(self.fname_template)

    def save(self, step, token):
        fname = self.fname_template.format(step, token)
        print('Saving checkpoint into %s...' % fname)
        outdict = {}
        for name, module in self.module_dict.items():
            outdict[name] = module.state_dict()
        torch.save(outdict, fname)

    def load(self, step, token, which=None, return_fname=False):
        fname = self.fname_template.format(step, token)
        if not os.path.exists(fname): print(f'WARNING: {fname} does not exist!')
        if return_fname: return fname
        print('Loading checkpoint from %s...' % fname)
        if torch.cuda.is_available():
            module_dict = torch.load(fname)
        else:
            module_dict = torch.load(fname, map_location=torch.device('cpu'))

        if which:
            module = self.module_dict[which]
            module.load_state_dict(module_dict[which])
            return

        for name, module in self.module_dict.items():
            module.load_state_dict(module_dict[name])

